/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1999
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *    Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: hostnames.c,v 1.5 2000/03/12 11:54:08 jordan Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "common-stuff.h"

#ifdef HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#else
# ifdef HAVE_SOCKET_H
#  include <socket.h>
# endif
#endif
#ifdef HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif
#ifdef HAVE_NETDB_H
# include <netdb.h>
#endif
#ifdef HAVE_ARPA_INET_H
# include <arpa/inet.h>
#endif

#include "hostnames.h"

/* ---------------------------------------------------------------------- *
 *                     private helpers                                    *
 * ---------------------------------------------------------------------- */

/* should be initialized in a threaded environment */
static const char *
get_myself
  (void)
{
  /* hold name for localhost */
  static char *myself;

  if (myself == 0) { 
    char buf [1024] ;
    if (gethostname (buf, sizeof (buf)) == 0 && buf [0])
      myself = XSTRDUP (buf) ;
  }
  
  return myself ;
}

/* upon parse error: set *is_error to 1 and return 0 */
static unsigned long
is_ip_pattern
  (const char *s, int *is_error)
{
  unsigned long result = 0 ;
  int a ;

  if ((a = strtol (s, (char**)&s, 10)) < 0 || a > 255 || s [0] != '.')
    goto parse_error;
  result = a ;
  
  if ((a = strtol (s+1, (char**)&s, 10)) < 0 || a > 255 || s [0] != '.')
    goto parse_error;
  result <<= 8 ;
  result  += a ;

  if ((a = strtol (s+1, (char**)&s, 10)) < 0 || a > 255 || s [0] != '.')
    goto parse_error;
  result <<= 8 ;
  result  += a ;

  if ((a = strtol (s+1, (char**)&s, 10)) < 0 || a > 255 || s [0] != '\0')
    goto parse_error;
  result <<= 8 ;
  result  += a ;
  
  return result ;

 parse_error:
  if (is_error != 0)
    *is_error = 1 ;
  return 0;
}

/* ---------------------------------------------------------------------- *
 *                     compare host names                                 *
 * ---------------------------------------------------------------------- */

const char *
get_my_host_name
   (void)
{
  return get_myself () ;
}

/* returns sort of a canonical host name */
const char *
get_host_ipaddress
   (const char *name)
{
  struct hostent *h;
  struct in_addr in;
  
  if (name == 0 && (name = get_myself ()) == 0)
    return 0 ;
  
  if (is_ip_pattern (name, 0))
    return name ;
  
  if ((h = gethostbyname (name)) == 0)
    return 0 ;

  /* convert to ip address */
  memcpy (&in.s_addr, *h->h_addr_list, h->h_length); 

  return inet_ntoa (in);
}

int
hostnomatch
  (const char *pattern,
   const char   *entry,
   size_t          len)
{
  return -1 ;
}


int
hostcmp
  (const char *pattern,
   const char   *entry,
   size_t          len)
{
  char buf [1024] ;
  unsigned long mask = -1 ;
  struct hostent *h;
  struct in_addr in1, in2;
  int n ;

  /* quick check for impossible hostnames */
  if (*entry == '#' || *entry == '@')
    return -1 ;

  /* this is easy, is it not ? */
  if (len <= 0) {
    if (strcasecmp (pattern, entry) == 0)
      return 0;   
    
  } else {
    /* now, the general case */
    char *p;
    if (strncasecmp (pattern, entry, len) == 0)
      return 0;
    
    /* need to cut the entry string from the general
       name@<entry-string> notation */
    if (len >= sizeof (buf) - 1)
      return -1 ; 
    memcpy (buf, entry, len) ;
    buf [len] = '\0' ;
    entry = buf ;

    /* check for <entry-string> ::= <ipnet>/<size> notation */
    if ((p = strchr (entry, '/')) != 0) {
      p [0] = '\0' ;
      /* /gen.eral.net.mask notation ? */
      if (strchr (p+1, '.') != 0) {
	if ((mask = is_ip_pattern (p+1, (n=0, &n))) == 0 && n)
	  return -1;
	
      } else {
	/* /<number-of-leading-bits> notation */
	if ((mask = strtol (p+1, &p, 10)) < 1 || mask > 32 || p [0] != '\0')
	  /* improper address notation */
	  return -1 ;
	mask = (mask < 32 ? (((((long)1)<<mask)-1)<<(32-mask)) : -1);
	/* convert to network notation */
      }
      mask = htonl (mask);
    }
  }

  /* resolve pattern adress, here - may not be thread safe */
  if ((h = gethostbyname (pattern)) == 0)
    return -1 ;
  memcpy (&in1.s_addr, *h->h_addr_list, h->h_length);

  /* mask off host bits => network match */
  in1.s_addr &= mask ;

  /* resolve entry, now - may not be thread safe */
  if ((h = gethostbyname (entry)) == 0)
    return -1 ;
  memcpy (&in2.s_addr, *h->h_addr_list, h->h_length); 

 /* for ipv4, in.s_addr is a 32 bit type, eg. unsigned int */
  return in1.s_addr - in2.s_addr ;
}
